{*********************************************************************************}
{
File Name.......: QRComps.zip
File Description: Implementation of DataSource property in QuickReport 2.0 components.
Targets.........: Delphi 3.
Author Name.....: George Vavoylogiannis
EMail...........: georgev@hol.gr
WEB.............: http://users.hol.gr/~georgev
File Status.....: Freeware
Category........: Database components.

### DESCRIPTION ###
Who said we don't need "DataSource" in QuickReport 2.0 components?
I need it so i created new derived ones adding the missing capability.

### NOTES ###
Free to use and redistribute, but my name must
appear somewhere in the source code, or in the software.
No warranty is given by the author, expressed or implied.

WARNING! THE CODE IS PROVIDED AS IS WITH NO GUARANTEES OF ANY KIND!
USE THIS AT YOUR OWN RISK - YOU ARE THE ONLY PERSON RESPONSIBLE FOR
ANY DAMAGE THIS CODE MAY CAUSE - YOU HAVE BEEN WARNED!

}
{**********************************************************************************}


{HISTORY
6/3/98 Initial realease of quickreport components. All components derived
       as T<quickreport component>.
       'DataSource' property on TTQuickReport and DB components via
       'TQRDataLink' (I DESPERTLY NEEDED DATASOURCE!!!).
       Also recovered TQRDBCalc .
07/10/98 'Datasource' property on TTQRSubDetail.
}
unit QRComps;

interface
uses Classes, TypInfo, Db, QuickRpt, Qrctrls, QRExtra;

type

 TTQRBand = class(TQRBand);
 TTQRChildBand = class(TQRChildBand);
 TTQRGroup = class(TQRGroup);
 TTQRLabel = class(TQRLabel);
 TTQRExpr = class(TQRExpr);
 TTQRSysData = class(TQRSysData);
 TTQRDBCalc = class (TQRDBCalc);
 TTQRMemo = class(TQRMemo);
 TTQRRichText = class(TQRRichText);
 TTQRShape = class(TQRShape);
 TTQRImage = class(TQRImage);

{ TQRDBTextDataLink }
 TQRDataLink = class (TDataLink)
 private
   FOwner: TComponent;
 protected
   procedure ActiveChanged; override;
 public
   constructor Create(AOwner: TComponent);
 end;

 { TTQuickReport }
 TTQuickRep = class (TQuickRep)
 private
   FDataLink: TQRDataLink;
   function GetDataSource: TDataSource;
   procedure SetDataSource(Value: TDataSource);
 public
   constructor Create(AOwner : TComponent); override;
   destructor Destroy; override;
 published
   property DataSource: TDataSource read GetDataSource write SetDataSource;
 end;

 { TTQRSubDetail }
 TTQRSubDetail = class(TQRSubDetail)
 private
   FDataLink: TQRDataLink;
   function GetDataSource: TDataSource;
   procedure SetDataSource(Value: TDataSource);
 public
   constructor Create(AOwner : TComponent); override;
   destructor Destroy; override;
 published
   property DataSource: TDataSource read GetDataSource write SetDataSource;
 end;

 { TTQRDBText }
 TTQRDBText = class (TQRDBText)
 private
   FDataLink: TQRDataLink;
   function GetDataSource: TDataSource;
   procedure SetDataSource(Value: TDataSource);
 public
   constructor Create(AOwner : TComponent); override;
   destructor Destroy; override;
 published
   property DataSource: TDataSource read GetDataSource write SetDataSource;
 end;

 { TTQRDBRichText }
 TTQRDBRichText = class (TQRDBRichText)
 private
   FDataLink: TQRDataLink;
   function GetDataSource: TDataSource;
   procedure SetDataSource(Value: TDataSource);
 public
   constructor Create(AOwner : TComponent); override;
   destructor Destroy; override;
 published
   property DataSource: TDataSource read GetDataSource write SetDataSource;
 end;

 { TTQRDBImage }
 TTQRDBImage = class (TQRDBImage)
 private
   FDataLink: TQRDataLink;
   function GetDataSource: TDataSource;
   procedure SetDataSource(Value: TDataSource);
 public
   constructor Create(AOwner : TComponent); override;
   destructor Destroy; override;
 published
   property DataSource: TDataSource read GetDataSource write SetDataSource;
 end;

procedure Register;

implementation
procedure Register;
begin
  RegisterComponents('QRControls', [ TTQuickRep,
                                     TTQRSubDetail,
                                     TTQRBand,
                                     TTQRChildBand,
                                     TTQRGroup,
                                     TTQRLabel,
                                     TTQRDBText,
                                     TTQRExpr,
                                     TTQRDBCalc,
                                     TTQRSysData,
                                     TTQRMemo,
                                     TTQRRichText,
                                     TTQRDBRichText,
                                     TTQRShape,
                                     TTQRImage,
                                     TTQRDBImage]);
end;

{ TQRDataLink }
constructor TQRDataLink.Create(AOwner: TComponent);
begin
 inherited Create;
 FOwner := AOwner;
end;

procedure TQRDataLink.ActiveChanged;
var
  PropInfo: PPropInfo;
begin
 inherited;
  PropInfo := TypInfo.GetPropInfo(FOwner.ClassInfo, 'DataSet');
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
    TypInfo.SetOrdProp(FOwner, PropInfo, LongInt(DataSet));
end;

{ TTQuickRep }
constructor TTQuickRep.Create(AOwner : TComponent);
begin
 inherited;
 FDataLink := TQRDataLink.Create(Self);
end;

destructor TTQuickRep.Destroy;
begin
 FDataLink.Free;
 inherited;
end;

function TTQuickRep.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTQuickRep.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
     begin
     Value.FreeNotification(Self);
     if Value.DataSet <> nil then
        DataSet := Value.DataSet;
     end;
end;

{ TTQRSubDetail }
constructor TTQRSubDetail.Create(AOwner : TComponent);
begin
 inherited;
 FDataLink := TQRDataLink.Create(Self);
end;

destructor TTQRSubDetail.Destroy;
begin
 FDataLink.Free;
 inherited;
end;

function TTQRSubDetail.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTQRSubDetail.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
     begin
     Value.FreeNotification(Self);
     if Value.DataSet <> nil then
        DataSet := Value.DataSet;
     end;
end;

{ TTQRDBText }
constructor TTQRDBText.Create(AOwner : TComponent);
begin
 inherited;
 FDataLink := TQRDataLink.Create(Self);
end;

destructor TTQRDBText.Destroy;
begin
 FDataLink.Free;
 inherited;
end;

function TTQRDBText.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTQRDBText.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
     begin
     Value.FreeNotification(Self);
     if Value.DataSet <> nil then
        DataSet := Value.DataSet;
     end;
end;

{ TTQRDBRichText }
constructor TTQRDBRichText.Create(AOwner : TComponent);
begin
 inherited;
 FDataLink := TQRDataLink.Create(Self);
end;

destructor TTQRDBRichText.Destroy;
begin
 FDataLink.Free;
 inherited;
end;

function TTQRDBRichText.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTQRDBRichText.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
     begin
     Value.FreeNotification(Self);
     if Value.DataSet <> nil then
        DataSet := Value.DataSet;
     end;
end;

{ TTQRDBCalc
constructor TTQRDBCalc.Create(AOwner : TComponent);
begin
 inherited;
 FDataLink := TQRDataLink.Create(Self);
end;

destructor TTQRDBCalc.Destroy;
begin
 FDataLink.Free;
 inherited;
end;

function TTQRDBCalc.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTQRDBCalc.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
     begin
     Value.FreeNotification(Self);
     if Value.DataSet <> nil then
        DataSet := Value.DataSet;
     end;
end;
}

{ TTQRDBImage }
constructor TTQRDBImage.Create(AOwner : TComponent);
begin
 inherited;
 FDataLink := TQRDataLink.Create(Self);
end;

destructor TTQRDBImage.Destroy;
begin
 FDataLink.Free;
 inherited;
end;

function TTQRDBImage.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TTQRDBImage.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then
     begin
     Value.FreeNotification(Self);
     if Value.DataSet <> nil then
        DataSet := Value.DataSet;
     end;
end;

end.
